@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2016  Jean-Philippe Lang
+# Copyright (C) 2006-2017  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,8 +16,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 class IssueRelationsController < ApplicationController
-  before_filter :find_issue, :authorize, :only => [:index, :create]
-  before_filter :find_relation, :only => [:show, :destroy]
+  helper :issues
+
+  before_action :find_issue, :authorize, :only => [:index, :create]
+  before_action :find_relation, :only => [:show, :destroy]
 
   accept_api_auth :index, :show, :create, :destroy
 
@@ -25,7 +27,7 @@ class IssueRelationsController < ApplicationController
     @relations = @issue.relations
 
     respond_to do |format|
-      format.html { render :nothing => true }
+      format.html { head 200 }
       format.api
     end
   end
@@ -34,17 +36,15 @@ class IssueRelationsController < ApplicationController
     raise Unauthorized unless @relation.visible?
 
     respond_to do |format|
-      format.html { render :nothing => true }
+      format.html { head 200 }
       format.api
     end
   end
 
   def create
-    @relation = IssueRelation.new(params[:relation])
+    @relation = IssueRelation.new
     @relation.issue_from = @issue
-    if params[:relation] && m = params[:relation][:issue_to_id].to_s.strip.match(/^#?(\d+)$/)
-      @relation.issue_to = Issue.visible.find_by_id(m[1].to_i)
-    end
+    @relation.safe_attributes = params[:relation]
     @relation.init_journals(User.current)
     saved = @relation.save
 
