@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2016  Jean-Philippe Lang
+# Copyright (C) 2006-2017  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@ class CustomValue < ActiveRecord::Base
   belongs_to :customized, :polymorphic => true
   attr_protected :id
 
+  after_save :custom_field_after_save_custom_value
+
   def initialize(attributes=nil, *args)
     super
     if new_record? && custom_field && !attributes.key?(:value)
@@ -40,6 +42,10 @@ class CustomValue < ActiveRecord::Base
     custom_field.visible?
   end
 
+  def attachments_visible?(user)
+    visible? && customized && customized.visible?(user)
+  end
+
   def required?
     custom_field.is_required?
   end
@@ -47,4 +53,10 @@ class CustomValue < ActiveRecord::Base
   def to_s
     value.to_s
   end
+
+  private
+
+  def custom_field_after_save_custom_value
+    custom_field.after_save_custom_value(self)
+  end
 end
