@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2016  Jean-Philippe Lang
+# Copyright (C) 2006-2017  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -32,11 +32,6 @@ class Issue < ActiveRecord::Base
   belongs_to :category, :class_name => 'IssueCategory'
 
   has_many :journals, :as => :journalized, :dependent => :destroy, :inverse_of => :journalized
-  has_many :visible_journals,
-    lambda {where(["(#{Journal.table_name}.private_notes = ? OR (#{Project.allowed_to_condition(User.current, :view_private_notes)}))", false])},
-    :class_name => 'Journal',
-    :as => :journalized
-
   has_many :time_entries, :dependent => :destroy
   has_and_belongs_to_many :changesets, lambda {order("#{Changeset.table_name}.committed_on ASC, #{Changeset.table_name}.id ASC")}
 
@@ -52,13 +47,14 @@ class Issue < ActiveRecord::Base
 
   acts_as_event :title => Proc.new {|o| "#{o.tracker.name} ##{o.id} (#{o.status}): #{o.subject}"},
                 :url => Proc.new {|o| {:controller => 'issues', :action => 'show', :id => o.id}},
-                :type => Proc.new {|o| 'issue' + (o.closed? ? ' closed' : '') }
+                :type => Proc.new {|o| 'issue' + (o.closed? ? '-closed' : '') }
 
   acts_as_activity_provider :scope => preload(:project, :author, :tracker, :status),
                             :author_key => :author_id
 
   DONE_RATIO_OPTIONS = %w(issue_field issue_status)
 
+  attr_accessor :deleted_attachment_ids
   attr_reader :current_journal
   delegate :notes, :notes=, :private_notes, :private_notes=, :to => :current_journal, :allow_nil => true
 
@@ -83,17 +79,17 @@ class Issue < ActiveRecord::Base
   scope :open, lambda {|*args|
     is_closed = args.size > 0 ? !args.first : false
     joins(:status).
-    where("#{IssueStatus.table_name}.is_closed = ?", is_closed)
+    where(:issue_statuses => {:is_closed => is_closed})
   }
 
-  scope :recently_updated, lambda { order("#{Issue.table_name}.updated_on DESC") }
+  scope :recently_updated, lambda { order(:updated_on => :desc) }
   scope :on_active_project, lambda {
     joins(:project).
-    where("#{Project.table_name}.status = ?", Project::STATUS_ACTIVE)
+    where(:projects => {:status => Project::STATUS_ACTIVE})
   }
   scope :fixed_version, lambda {|versions|
     ids = [versions].flatten.compact.map {|v| v.is_a?(Version) ? v.id : v}
-    ids.any? ? where(:fixed_version_id => ids) : where('1=0')
+    ids.any? ? where(:fixed_version_id => ids) : none
   }
   scope :assigned_to, lambda {|arg|
     arg = Array(arg).uniq
@@ -102,6 +98,12 @@ class Issue < ActiveRecord::Base
     ids.compact!
     ids.any? ? where(:assigned_to_id => ids) : none
   }
+  scope :like, lambda {|q|
+    q = q.to_s
+    if q.present?
+      where("LOWER(#{table_name}.subject) LIKE LOWER(?)", "%#{q}%")
+    end
+  }
 
   before_validation :default_assign, on: :create
   before_validation :clear_disabled_fields
@@ -109,7 +111,7 @@ class Issue < ActiveRecord::Base
               :force_updated_on_change, :update_closed_on, :set_assigned_to_was
   after_save {|issue| issue.send :after_project_change if !issue.id_changed? && issue.project_id_changed?}
   after_save :reschedule_following_issues, :update_nested_set_attributes,
-             :update_parent_attributes, :create_journal
+             :update_parent_attributes, :delete_selected_attachments, :create_journal
   # Should be after_create but would be called before previous after_save callbacks
   after_save :after_create_from_copy
   after_destroy :update_parent_attributes
@@ -243,6 +245,8 @@ class Issue < ActiveRecord::Base
     @spent_hours = nil
     @total_spent_hours = nil
     @total_estimated_hours = nil
+    @last_updated_by = nil
+    @last_notes = nil
     base_reload(*args)
   end
 
@@ -261,15 +265,20 @@ class Issue < ActiveRecord::Base
   # Copies attributes from another issue, arg can be an id or an Issue
   def copy_from(arg, options={})
     issue = arg.is_a?(Issue) ? arg : Issue.visible.find(arg)
-    self.attributes = issue.attributes.dup.except("id", "root_id", "parent_id", "lft", "rgt", "created_on", "updated_on", "closed_on")
+    self.attributes = issue.attributes.dup.except("id", "root_id", "parent_id", "lft", "rgt", "created_on", "updated_on", "status_id", "closed_on")
     self.custom_field_values = issue.custom_field_values.inject({}) {|h,v| h[v.custom_field_id] = v.value; h}
-    self.status = issue.status
+    if options[:keep_status]
+      self.status = issue.status
+    end
     self.author = User.current
     unless options[:attachments] == false
       self.attachments = issue.attachments.map do |attachement|
         attachement.copy(:container => self)
       end
     end
+    unless options[:watchers] == false
+      self.watcher_user_ids = issue.watcher_user_ids.dup
+    end
     @copied_from = issue
     @copy_options = options
     self
@@ -366,6 +375,9 @@ class Issue < ActiveRecord::Base
   def project=(project, keep_tracker=false)
     project_was = self.project
     association(:project).writer(project)
+    if project != project_was
+      @safe_attribute_names = nil
+    end
     if project_was && project && project_was != project
       @assignable_versions = nil
 
@@ -376,6 +388,11 @@ class Issue < ActiveRecord::Base
       if category
         self.category = project.issue_categories.find_by_name(category.name)
       end
+      # Clear the assignee if not available in the new project for new issues (eg. copy)
+      # For existing issue, the previous assignee is still valid, so we keep it
+      if new_record? && assigned_to && !assignable_users.include?(assigned_to)
+        self.assigned_to_id = nil
+      end
       # Keep the fixed_version if it's still valid in the new_project
       if fixed_version && fixed_version.project != project && !project.shared_versions.include?(fixed_version)
         self.fixed_version = nil
@@ -403,8 +420,12 @@ class Issue < ActiveRecord::Base
     write_attribute(:description, arg)
   end
 
+  def deleted_attachment_ids
+    Array(@deleted_attachment_ids).map(&:to_i)
+  end
+
   # Overrides assign_attributes so that project and tracker get assigned first
-  def assign_attributes_with_project_and_tracker_first(new_attributes, *args)
+  def assign_attributes(new_attributes, *args)
     return if new_attributes.nil?
     attrs = new_attributes.dup
     attrs.stringify_keys!
@@ -414,10 +435,8 @@ class Issue < ActiveRecord::Base
         send "#{attr}=", attrs.delete(attr)
       end
     end
-    send :assign_attributes_without_project_and_tracker_first, attrs, *args
+    super attrs, *args
   end
-  # Do not redefine alias chain on reload (see #4838)
-  alias_method_chain(:assign_attributes, :project_and_tracker_first) unless method_defined?(:assign_attributes_without_project_and_tracker_first)
 
   def attributes=(new_attributes)
     assign_attributes new_attributes
@@ -465,6 +484,9 @@ class Issue < ActiveRecord::Base
     :if => lambda {|issue, user| (issue.new_record? || issue.attributes_editable?(user)) &&
       user.allowed_to?(:manage_subtasks, issue.project)}
 
+  safe_attributes 'deleted_attachment_ids',
+    :if => lambda {|issue, user| issue.attachments_deletable?(user)}
+
   def safe_attribute_names(user=nil)
     names = super
     names -= disabled_core_fields
@@ -517,10 +539,17 @@ class Issue < ActiveRecord::Base
         self.tracker_id = t
       end
     end
-    if project
+    if project && tracker.nil?
       # Set a default tracker to accept custom field values
       # even if tracker is not specified
-      self.tracker ||= allowed_target_trackers(user).first
+      allowed_trackers = allowed_target_trackers(user)
+
+      if attrs['parent_issue_id'].present?
+        # If parent_issue_id is present, the first tracker for which this field
+        # is not disabled is chosen as default
+        self.tracker = allowed_trackers.detect {|t| t.core_fields.include?('parent_issue_id')}
+      end
+      self.tracker ||= allowed_trackers.first
     end
 
     statuses_allowed = new_statuses_allowed_to(user)
@@ -533,14 +562,7 @@ class Issue < ActiveRecord::Base
       self.status = statuses_allowed.first || default_status
     end
     if (u = attrs.delete('assigned_to_id')) && safe_attribute?('assigned_to_id')
-      if u.blank?
-        self.assigned_to_id = nil
-      else
-        u = u.to_i
-        if assignable_users.any?{|assignable_user| assignable_user.id == u}
-          self.assigned_to_id = u
-        end
-      end
+      self.assigned_to_id = u
     end
 
 
@@ -703,6 +725,12 @@ class Issue < ActiveRecord::Base
       end
     end
 
+    if assigned_to_id_changed? && assigned_to_id.present?
+      unless assignable_users.include?(assigned_to)
+        errors.add :assigned_to_id, :invalid
+      end
+    end
+
     # Checks parent issue assignment
     if @invalid_parent_issue_id.present?
       errors.add :parent_issue_id, :invalid
@@ -714,6 +742,9 @@ class Issue < ActiveRecord::Base
           @parent_issue.self_and_ancestors.any? {|a| a.relations_from.any? {|r| r.relation_type == IssueRelation::TYPE_PRECEDES && r.issue_to.would_reschedule?(self)}}
         )
         errors.add :parent_issue_id, :invalid
+      elsif !closed? && @parent_issue.closed?
+        # cannot attach an open issue to a closed parent
+        errors.add :base, :open_issue_with_closed_parent
       elsif !new_record?
         # moving an existing issue
         if move_possible?(@parent_issue)
@@ -782,6 +813,11 @@ class Issue < ActiveRecord::Base
     @current_journal
   end
 
+  # Clears the current journal
+  def clear_journal
+    @current_journal = nil
+  end
+
   # Returns the names of attributes that are journalized when updating the issue
   def journalized_attribute_names
     names = Issue.column_names - %w(id root_id lft rgt lock_version created_on updated_on closed_on)
@@ -809,6 +845,26 @@ class Issue < ActiveRecord::Base
     scope
   end
 
+  # Returns the journals that are visible to user with their index
+  # Used to display the issue history
+  def visible_journals_with_index(user=User.current)
+    result = journals.
+      preload(:details).
+      preload(:user => :email_address).
+      reorder(:created_on, :id).to_a
+
+    result.each_with_index {|j,i| j.indice = i+1}
+
+    unless user.allowed_to?(:view_private_notes, project)
+      result.select! do |journal|
+        !journal.private_notes? || journal.user == user
+      end
+    end
+    Journal.preload_journals_details_custom_fields(result)
+    result.select! {|journal| journal.notes? || journal.visible_details.any?}
+    result
+  end
+
   # Returns the initial status of the issue
   # Returns nil for a new issue
   def status_was
@@ -871,7 +927,9 @@ class Issue < ActiveRecord::Base
   def assignable_users
     users = project.assignable_users(tracker).to_a
     users << author if author && author.active?
-    users << assigned_to if assigned_to
+    if assigned_to_id_was.present? && assignee = Principal.find_by_id(assigned_to_id_was)
+      users << assignee
+    end
     users.uniq.sort
   end
 
@@ -907,44 +965,46 @@ class Issue < ActiveRecord::Base
 
   # Returns an array of statuses that user is able to apply
   def new_statuses_allowed_to(user=User.current, include_default=false)
-    if new_record? && @copied_from
-      [default_status, @copied_from.status].compact.uniq.sort
-    else
-      initial_status = nil
-      if new_record?
-        # nop
-      elsif tracker_id_changed?
-        if Tracker.where(:id => tracker_id_was, :default_status_id => status_id_was).any?
-          initial_status = default_status
-        elsif tracker.issue_status_ids.include?(status_id_was)
-          initial_status = IssueStatus.find_by_id(status_id_was)
-        else
-          initial_status = default_status
-        end
+    initial_status = nil
+    if new_record?
+      # nop
+    elsif tracker_id_changed?
+      if Tracker.where(:id => tracker_id_was, :default_status_id => status_id_was).any?
+        initial_status = default_status
+      elsif tracker.issue_status_ids.include?(status_id_was)
+        initial_status = IssueStatus.find_by_id(status_id_was)
       else
-        initial_status = status_was
+        initial_status = default_status
       end
-
-      initial_assigned_to_id = assigned_to_id_changed? ? assigned_to_id_was : assigned_to_id
-      assignee_transitions_allowed = initial_assigned_to_id.present? &&
-        (user.id == initial_assigned_to_id || user.group_ids.include?(initial_assigned_to_id))
-
-      statuses = []
-      statuses += IssueStatus.new_statuses_allowed(
-        initial_status,
-        user.admin ? Role.all.to_a : user.roles_for_project(project),
-        tracker,
-        author == user,
-        assignee_transitions_allowed
-      )
-      statuses << initial_status unless statuses.empty?
-      statuses << default_status if include_default || (new_record? && statuses.empty?)
-      statuses = statuses.compact.uniq.sort
-      if blocked?
-        statuses.reject!(&:is_closed?)
-      end
-      statuses
+    else
+      initial_status = status_was
     end
+
+    initial_assigned_to_id = assigned_to_id_changed? ? assigned_to_id_was : assigned_to_id
+    assignee_transitions_allowed = initial_assigned_to_id.present? &&
+      (user.id == initial_assigned_to_id || user.group_ids.include?(initial_assigned_to_id))
+
+    statuses = []
+    statuses += IssueStatus.new_statuses_allowed(
+      initial_status,
+      user.admin ? Role.all.to_a : user.roles_for_project(project),
+      tracker,
+      author == user,
+      assignee_transitions_allowed
+    )
+    statuses << initial_status unless statuses.empty?
+    statuses << default_status if include_default || (new_record? && statuses.empty?)
+
+    statuses = statuses.compact.uniq.sort
+    if blocked? || descendants.open.any?
+      # cannot close a blocked issue or a parent with open subtasks
+      statuses.reject!(&:is_closed?)
+    end
+    if ancestors.open(false).any?
+      # cannot reopen a subtask of a closed parent
+      statuses.select!(&:is_closed?)
+    end
+    statuses
   end
 
   # Returns the previous assignee (user or group) if changed
@@ -1036,6 +1096,22 @@ class Issue < ActiveRecord::Base
     @relations ||= IssueRelation::Relations.new(self, (relations_from + relations_to).sort)
   end
 
+  def last_updated_by
+    if @last_updated_by
+      @last_updated_by.presence
+    else
+      journals.reorder(:id => :desc).first.try(:user)
+    end
+  end
+
+  def last_notes
+    if @last_notes
+      @last_notes
+    else
+      journals.where.not(notes: '').reorder(:id => :desc).first.try(:notes)
+    end
+  end
+
   # Preloads relations for a collection of issues
   def self.load_relations(issues)
     if issues.any?
@@ -1099,6 +1175,45 @@ class Issue < ActiveRecord::Base
       where(:ancestors => {:id => issues.map(&:id)})
   end
 
+  # Preloads users who updated last a collection of issues
+  def self.load_visible_last_updated_by(issues, user=User.current)
+    if issues.any?
+      issue_ids = issues.map(&:id)
+      journal_ids = Journal.joins(issue: :project).
+        where(:journalized_type => 'Issue', :journalized_id => issue_ids).
+        where(Journal.visible_notes_condition(user, :skip_pre_condition => true)).
+        group(:journalized_id).
+        maximum(:id).
+        values
+      journals = Journal.where(:id => journal_ids).preload(:user).to_a
+
+      issues.each do |issue|
+        journal = journals.detect {|j| j.journalized_id == issue.id}
+        issue.instance_variable_set("@last_updated_by", journal.try(:user) || '')
+      end
+    end
+  end
+
+  # Preloads visible last notes for a collection of issues
+  def self.load_visible_last_notes(issues, user=User.current)
+    if issues.any?
+      issue_ids = issues.map(&:id)
+      journal_ids = Journal.joins(issue: :project).
+        where(:journalized_type => 'Issue', :journalized_id => issue_ids).
+        where(Journal.visible_notes_condition(user, :skip_pre_condition => true)).
+        where.not(notes: '').
+        group(:journalized_id).
+        maximum(:id).
+        values
+      journals = Journal.where(:id => journal_ids).to_a
+
+      issues.each do |issue|
+        journal = journals.detect {|j| j.journalized_id == issue.id}
+        issue.instance_variable_set("@last_notes", journal.try(:notes) || '')
+      end
+    end
+  end
+
   # Finds an issue relation given its id.
   def find_relation(relation_id)
     IssueRelation.where("issue_to_id = ? OR issue_from_id = ?", id, id).find(relation_id)
@@ -1165,7 +1280,8 @@ class Issue < ActiveRecord::Base
 
   def soonest_start(reload=false)
     if @soonest_start.nil? || reload
-      dates = relations_to(reload).collect{|relation| relation.successor_soonest_start}
+      relations_to.reload if reload
+      dates = relations_to.collect{|relation| relation.successor_soonest_start}
       p = @parent_issue || parent
       if p && Setting.parent_issue_dates == 'derived'
         dates << p.soonest_start
@@ -1475,6 +1591,7 @@ class Issue < ActiveRecord::Base
       # Change project and keep project
       child.send :project=, project, true
       unless child.save
+        errors.add :base, l(:error_move_of_child_not_possible, :child => "##{child.id}", :errors => child.errors.full_messages.join(", "))
         raise ActiveRecord::Rollback
       end
     end
@@ -1575,18 +1692,23 @@ class Issue < ActiveRecord::Base
       end
 
       if p.done_ratio_derived?
-        # done ratio = weighted average ratio of leaves
+        # done ratio = average ratio of children weighted with their total estimated hours
         unless Issue.use_status_for_done_ratio? && p.status && p.status.default_done_ratio
-          child_count = p.children.count
-          if child_count > 0
-            average = p.children.where("estimated_hours > 0").average(:estimated_hours).to_f
-            if average == 0
-              average = 1
+          children = p.children.to_a
+          if children.any?
+            child_with_total_estimated_hours = children.select {|c| c.total_estimated_hours.to_f > 0.0}
+            if child_with_total_estimated_hours.any?
+              average = child_with_total_estimated_hours.map(&:total_estimated_hours).sum.to_f / child_with_total_estimated_hours.count
+            else
+              average = 1.0
             end
-            done = p.children.joins(:status).
-              sum("COALESCE(CASE WHEN estimated_hours > 0 THEN estimated_hours ELSE NULL END, #{average}) " +
-                  "* (CASE WHEN is_closed = #{self.class.connection.quoted_true} THEN 100 ELSE COALESCE(done_ratio, 0) END)").to_f
-            progress = done / (average * child_count)
+            done = children.map {|c|
+              estimated = c.total_estimated_hours.to_f
+              estimated = average unless estimated > 0.0
+              ratio = c.closed? ? 100 : (c.done_ratio || 0)
+              estimated * ratio
+            }.sum
+            progress = done / (average * children.count)
             p.done_ratio = progress.round
           end
         end
@@ -1616,6 +1738,13 @@ class Issue < ActiveRecord::Base
     end
   end
 
+  def delete_selected_attachments
+    if deleted_attachment_ids.present?
+      objects = attachments.where(:id => deleted_attachment_ids.map(&:to_i))
+      attachments.delete(objects)
+    end
+  end
+
   # Callback on file attachment
   def attachment_added(attachment)
     if current_journal && !attachment.new_record?
@@ -1647,10 +1776,14 @@ class Issue < ActiveRecord::Base
     end
   end
 
-  # Default assignment based on category
+  # Default assignment based on project or category
   def default_assign
-    if assigned_to.nil? && category && category.assigned_to
-      self.assigned_to = category.assigned_to
+    if assigned_to.nil?
+      if category && category.assigned_to
+        self.assigned_to = category.assigned_to
+      elsif project && project.default_assigned_to
+        self.assigned_to = project.default_assigned_to
+      end
     end
   end
 
