@@ -1,7 +1,7 @@
 # encoding: utf-8
 #
 # Redmine - project management software
-# Copyright (C) 2006-2016  Jean-Philippe Lang
+# Copyright (C) 2006-2017  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,13 +19,13 @@
 
 require File.expand_path('../../../test_helper', __FILE__)
 
-class ApplicationHelperTest < ActionView::TestCase
-  include Redmine::I18n
+class ApplicationHelperTest < Redmine::HelperTest
   include ERB::Util
   include Rails.application.routes.url_helpers
 
-  fixtures :projects, :roles, :enabled_modules, :users,
-           :email_addresses,
+  fixtures :projects, :enabled_modules,
+           :users, :email_addresses,
+           :members, :member_roles, :roles,
            :repositories, :changesets,
            :projects_trackers,
            :trackers, :issue_statuses, :issues, :versions, :documents,
@@ -170,6 +170,12 @@ RAW
     end
   end
 
+  def test_attached_images_with_hires_naming
+    attachment = Attachment.generate!(:filename => 'image@2x.png')
+      assert_equal %(<p><img src="/attachments/download/#{attachment.id}/image@2x.png" srcset="/attachments/download/#{attachment.id}/image@2x.png 2x" alt="" /></p>),
+        textilizable("!image@2x.png!", :attachments => [attachment])
+  end
+
   def test_attached_images_filename_extension
     set_tmp_attachments_directory
     a1 = Attachment.new(
@@ -377,12 +383,26 @@ RAW
       # invalid expressions
       'source:'                     => 'source:',
       # url hash
-      "http://foo.bar/FAQ#3"       => '<a class="external" href="http://foo.bar/FAQ#3">http://foo.bar/FAQ#3</a>',
+      "http://foo.bar/FAQ#3"        => '<a class="external" href="http://foo.bar/FAQ#3">http://foo.bar/FAQ#3</a>',
+      # user
+      'user:jsmith'                 => link_to_user(User.find_by_id(2)),
+      'user#2'                      => link_to_user(User.find_by_id(2)),
+      '@jsmith'                     => link_to_user(User.find_by_id(2)),
+      # invalid user
+      'user:foobar'                 => 'user:foobar',
     }
     @project = Project.find(1)
     to_test.each { |text, result| assert_equal "<p>#{result}</p>", textilizable(text), "#{text} failed" }
   end
 
+  def test_user_links_with_email_as_login_name_should_not_be_parsed
+    u = User.generate!(:login => 'jsmith@somenet.foo')
+    raw = "@jsmith@somenet.foo should not be parsed in jsmith@somenet.foo"
+
+    assert_match %r{<p><a class="user active".*>#{u.name}</a> should not be parsed in <a class="email" href="mailto:jsmith@somenet.foo">jsmith@somenet.foo</a></p>},
+      textilizable(raw, :project => Project.find(1))
+  end
+
   def test_should_not_parse_redmine_links_inside_link
     raw = "r1 should not be parsed in http://example.com/url-r1/"
     assert_match %r{<p><a class="changeset".*>r1</a> should not be parsed in <a class="external" href="http://example.com/url-r1/">http://example.com/url-r1/</a></p>},
@@ -647,7 +667,7 @@ RAW
 
   def test_attachment_links
     text = 'attachment:error281.txt'
-    result = link_to("error281.txt", "/attachments/download/1/error281.txt",
+    result = link_to("error281.txt", "/attachments/1/error281.txt",
                      :class => "attachment")
     assert_equal "<p>#{result}</p>",
                  textilizable(text,
@@ -659,7 +679,7 @@ RAW
     set_tmp_attachments_directory
     a1 = Attachment.generate!(:filename => "test.txt", :created_on => 1.hour.ago)
     a2 = Attachment.generate!(:filename => "test.txt")
-    result = link_to("test.txt", "/attachments/download/#{a2.id}/test.txt",
+    result = link_to("test.txt", "/attachments/#{a2.id}/test.txt",
                      :class => "attachment")
     assert_equal "<p>#{result}</p>",
                  textilizable('attachment:test.txt', :attachments => [a1, a2])
@@ -1096,6 +1116,8 @@ EXPECTED
   end
 
   def test_table_of_content
+    set_language_if_valid 'en'
+
     raw = <<-RAW
 {{toc}}
 
@@ -1128,6 +1150,7 @@ h2. "Project Name !/attachments/1234/logo_small.gif! !/attachments/5678/logo_2.p
 RAW
 
     expected =  '<ul class="toc">' +
+                  '<li><strong>Table of contents</strong></li>' +
                   '<li><a href="#Title">Title</a>' +
                     '<ul>' +
                       '<li><a href="#Subtitle-with-a-Wiki-link">Subtitle with a Wiki link</a></li>' +
@@ -1157,6 +1180,8 @@ RAW
   end
 
   def test_table_of_content_should_generate_unique_anchors
+    set_language_if_valid 'en'
+
     raw = <<-RAW
 {{toc}}
 
@@ -1168,6 +1193,7 @@ h2. Subtitle
 RAW
 
     expected =  '<ul class="toc">' +
+                  '<li><strong>Table of contents</strong></li>' +
                   '<li><a href="#Title">Title</a>' +
                     '<ul>' +
                       '<li><a href="#Subtitle">Subtitle</a></li>' +
@@ -1184,6 +1210,8 @@ RAW
   end
 
   def test_table_of_content_should_contain_included_page_headings
+    set_language_if_valid 'en'
+
     raw = <<-RAW
 {{toc}}
 
@@ -1193,6 +1221,7 @@ h1. Included
 RAW
 
     expected = '<ul class="toc">' +
+               '<li><strong>Table of contents</strong></li>' +
                '<li><a href="#Included">Included</a></li>' +
                '<li><a href="#Child-page-1">Child page 1</a></li>' +
                '</ul>'
@@ -1268,6 +1297,14 @@ RAW
     end
   end
 
+  def test_textilizable_with_formatting_set_to_false_should_not_format_text
+    assert_equal '*text*', textilizable("*text*", :formatting => false)
+  end
+
+  def test_textilizable_with_formatting_set_to_true_should_format_text
+    assert_equal '<p><strong>text</strong></p>', textilizable("*text*", :formatting => true)
+  end
+
   def test_parse_redmine_links_should_handle_a_tag_without_attributes
     text = '<a>http://example.com</a>'
     expected = text.dup
@@ -1543,4 +1580,29 @@ RAW
     assert_equal "#{ja} #{ja}...", result
     assert !result.html_safe?
   end
+
+  def test_back_url_should_remove_utf8_checkmark_from_referer
+    stubs(:request).returns(stub(:env => {'HTTP_REFERER' => "/path?utf8=\u2713&foo=bar"}))
+    assert_equal "/path?foo=bar", back_url
+  end
+
+  def test_hours_formatting
+    set_language_if_valid 'en'
+
+    with_settings :timespan_format => 'minutes' do
+      assert_equal '0:45', format_hours(0.75)
+      assert_equal '0:45 h', l_hours_short(0.75)
+      assert_equal '0:45 hour', l_hours(0.75)
+    end
+    with_settings :timespan_format => 'decimal' do
+      assert_equal '0.75', format_hours(0.75)
+      assert_equal '0.75 h', l_hours_short(0.75)
+      assert_equal '0.75 hour', l_hours(0.75)
+    end
+  end
+
+  def test_html_hours
+    assert_equal '<span class="hours hours-int">0</span><span class="hours hours-dec">:45</span>', html_hours('0:45')
+    assert_equal '<span class="hours hours-int">0</span><span class="hours hours-dec">.75</span>', html_hours('0.75')
+  end
 end
